#!/bin/sh
# *
# *    boostrap.sh - This file is part of Critecia
# *    Copyright (C) 2011 Christian A. Reiter
# *
# *    This program is free software: you can redistribute it and/or modify
# *    it under the terms of the GNU Affero General Public License as
# *    published by the Free Software Foundation, either version 3 of the
# *    License, or (at your option) any later version.
# *
# *    This program is distributed in the hope that it will be useful,
# *    but WITHOUT ANY WARRANTY; without even the implied warranty of
# *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# *    GNU Affero General Public License for more details.
# *
# *    You should have received a copy of the GNU Affero General Public License
# *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
# *


################## config values ##################

test_salt="DYhG93b0qyJfIxfs2guAiMubWwvniR2G0FgaC9mi"
test_cipher="76859309657451942496749683645"

return_var=""

conf_dir="../src/app/config"
database_php="$conf_dir/database.php"
core_php="$conf_dir/core.php"

sql_bootstrap="bootstrap.sql"
sql_basedata="basedata.sql"
sql_demodata="demodata.sql"

os=${OSTYPE//[0-9.]/}
if [ "$os" == "linux-gnu" ]; then
  mysql=`which mysql`;
elif [ "$os" == "darwin" ]; then
  mysql='/Applications/MAMP/Library/bin/mysql';
fi


################## functions ##################

# prints output <strong>
__echo(){
	echo "\033[1;37m$1\033[0m"
}



# prints prompt and default value in brackets, gets variable vom user and saves it in global $return_var
# $1: text to display
# $2: default value if user gives nothing
getvar() {
	# default value given?
	[ "$2" != "" ] && default="[$2]" || default=""
	echo -n "$1 $default: "
	read value
	if [ "$value" = "" ]; then
		return_var="$2"
	else
		return_var="$value"
	fi
}


# params: $1=filename, $2=key, $3=new_value
# 	sets new value in PHP array with given key in given file
# 	use with care! No tests!
change_array_value() { 
	sed -i "s/\(.*'$2' *=> *'\)\(.*\)\(',.*$\)/\1$3\3/" $1
# 	cat "$1" | sed "s/\(.*'$2' *=> *'\)\(.*\)\(',.*$\)/\1$3\3/" | grep -v "\*"
}


# params: $1=filename, $2=key, $3=new_value
# 	sets new value in PHP array with given key in given file
# 	use with care! No tests!
change_configure_value() { 
	sed -i "s/\(.*'$2'.*,.*'\)\(.*\)\(');$\)/\1$3\3/" $1
# 	cat "$1" | sed "s/\(.*'$2' *=> *'\)\(.*\)\(',.*$\)/\1$3\3/" | grep "$2"
}




################## main program ##################

__echo "Please make sure that a proper LAMP/MAMP/XAMP is installed."

#============== set up config files ==============

#-------------- database.php --------------

__echo "\nSetting up database environment..."
echo "    If you just press ENTER the default (in square brackets) will be used.\n"
getvar "	please enter hostname" "localhost"
hostname=$return_var

getvar "	please enter database name" "critecia"
database=$return_var

getvar "	please enter database user" "critecia"
database_user=$return_var

getvar "	please enter database password" "critecia"
database_password=$return_var

echo
echo -n "	Hostname: "; __echo "$hostname"
echo -n "	Database name: "; __echo "$database"
echo -n "	Database user: "; __echo "$database_user"
echo -n "	Database password: "; __echo "$database_password"
echo
echo "   If these values are correct, press \033[1;32mEnter\033[0m to set the values into the config files."
echo "   If not, press \033[1;31mCtrl+C\033[0m to stop the program now!"
read value

# copy database.php.default into place
cp "${database_php}.default" "$database_php"


# replace values in database.php
change_array_value "$database_php" "host" "$hostname"
change_array_value "$database_php" "database" "$database"
change_array_value "$database_php" "login" "$database_user"
change_array_value "$database_php" "password" "$database_password"

# grep -v "\*" "$database_php"
echo "\033[1;32mConfig files are now set up.\033[0m\n\n"


#-------------- core.php --------------

# copy core.php.default into place
cp "${core_php}.default" "$core_php"

change_configure_value "$core_php" "Security.salt" "$test_salt"
change_configure_value "$core_php" "Security.cipherSeed" "$test_cipher"




#============== directory permissions ==============

chmod -R o+w ../src/app/tmp





#============== database setup ==============

#-------------- create database --------------
db_created=0
__echo "The bootstrap script will now try to create a database 'critecia' and a user 'critecia' with all rights on it."
__echo "You need to enter the MYSQL root password for this action."
echo
echo "  If you want to set up the database and fill it with default values now press \033[1;32mEnter\033[0m now."
echo "  If not, press \033[1;31mCtrl-C\033[0m to stop the program now!"
read YN
for i in 1 2 3; do
	$mysql -u root -p < "$sql_bootstrap"
	if [ "$?" = 0 ]; then
		echo "\033[1;32mDatabase created successfully.\033[0m"
		db_created=1
		break;
	fi
done
if [ "$db_created" != 1 ]; then
	echo "\033[1;31mError - could not create database. Stopping script.\033[0m"
	exit 1
fi

#-------------- create schema using cake --------------

__echo "Trying to set up DB schema using cake console"
# I must be in the right dir for cake
cd ../src 
cake="cake/console/cake"

if [ -f "$cake" ]; then
	__echo "Found cake console."
	echo "  Please press \033[1;32mEnter\033[0m during the questions twice."
else
	echo "\033[1;31mError - could not find cake console. Stopping script.\033[0m"
	exit 1
fi




$cake schema create
if [ $? != 0 ]; then
	exit 1
fi

# go back into original directory
cd ../bootstrap

#-------------- create base data in database --------------
db_created=0
__echo "\nThe bootstrap script will now try to create base data in the database."
for i in 1 2 3; do
	mysql -u "$database_user" --password="$database_password" < "$sql_basedata"
	if [ "$?" = 0 ]; then
		echo "\033[1;32mBase data created successfully.\033[0m"
		db_created=1
		break;
	fi
done
if [ "$db_created" != 1 ]; then
	echo "\033[1;31mError - could not create base data. Stopping script.\033[0m"
	exit 1
fi

echo "\n\n"
__echo "Three important things are left to do now: Make sure that..."
echo "  ...your \033[1;32mApache has mod_rewrite enabled\033[0m"
echo "  ...your \033[1;32mApache config includes an 'AllowOverride All' for your www_doc directory\033[0m"
echo "  ...there exists a \033[1;32msymlink/alias in your www_doc to the critecia/src directory\033[0m"
__echo "\nhave fun...\n"
