<?php
/**
 *    AppController.php - This file is part of Critecia
 *    Copyright (C) 2011 Alexander Lang & Christian A. Reiter
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Affero General Public License as
 *    published by the Free Software Foundation, either version 3 of the
 *    License, or (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Affero General Public License for more details.
 *
 *    You should have received a copy of the GNU Affero General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

class AppController extends Controller {
	
	public $uses = array('User');
	public $helpers = array('Form', 'Html', 'Js' => array('Jquery'), 'Session', 'AccessControl');
 	public $components = array('Auth', 'Session', 'AccessControl');
	
	function beforeFilter() {
		parent::beforeFilter();
		
		// set LOCALE to German
		setlocale(LC_ALL, 'de');
		
		// Authentication setup
		$this->Auth->authenticate = array('Form');
		
		$this->Auth->authError = "Fehler bei der Authentifizierung.<br/>Bitte einloggen!";
		$this->Auth->loginError = 'Falscher Benutzername oder Passwort!';
		$this->Auth->userScope = array('User.active' => 1);
		$this->Auth->loginAction = array('global' => true, 'controller' => 'users', 'action' => 'login');
		$this->Auth->loginRedirect = array('global' => true, 'controller' => 'maintenances', 'action' => 'welcome');
		
		// Authorization setup
		$this->Auth->authorize = array('Controller');
		$this->Auth->allow('global_login', 'global_logout');
		
// 		debug($this->name . '/' . $this->action);

		if ($this->Auth->user('id')) {
// 			debug($this->Session->read('mode'));
			// extra behaviour for login/logout: they are NOT hospital specific, but NOT global for convenience
			if($this->name == 'Users' && in_array($this->action, array(
					'global_login', 'global_logout'))) {
				// just return and proceed with action.
				return;
			}
			
			// check every 60 seconds for new grants
			$now = time();
			$timeDifference = $now - $this->Session->read('LastUpdateTimestamp') ;
			if($timeDifference >= 60) {
				$this->Session->write('LastUpdateTimestamp', $now);
				$this->updateUserGrants();
			}

			$this->updateActiveHospital();

			if(isset($this->params['prefix']) and $this->params['prefix'] == 'global') {
				// global controller/action
				$this->set('global', true);
			} else {
				// hospital specific controller/action
				$this->set('global', false);

				if(!$this->current_hospital) { // no current_hospital selected at the moment!
					//FIXME: do this as Exception as CakePHP 2.0 requires this
					// or, as it maybe is no real ERROR, just check and respond to it.
					#$this->cakeError('hospitalRequired');
				}
			}
			$this->set('user', $this->Session->read('Auth.User'));
			$this->set('current_mode', $this->Session->read('mode'));
			$this->set('accessible_hospitals',
				$this->Session->read('Auth.User.AccessibleHospitals'));
		} else { // noone logged in
			$this->set('user', null);
			$this->set('current_mode', null);
			$this->set('accessible_hospitals', null);
		}
	}	
	
	
	/**
	 * checks if the current user is allowed th access the given URL
	 * @param $user mixed user that is checked
	 */
	function isAuthorized($user) {
		if( $this->AccessControl->check($this->name . '/' . $this->action))
			return true;
		return false;
    }
    
    
    /**
	 * switches global application mode 
	 * @param $mode string new mode - should be one of 'management', 'rotation', 'vacations', 'system', 'install'
	 */
	protected function switchMode($mode) {
		$default_mode = 'management';
		if ($this->Session->valid()) {
			if(!$mode) {
				$mode = $default_mode;
			}
			$global = false;
			if(in_array($mode, array('management', 'rotation', 'vacations', 'system', 'install'))) {
				$this->Session->write('mode', $mode);
				$this->set('current_mode',$mode);
				if(in_array($mode, array('install', 'system'))) {
					$global = true;
				}
			} else {
				$this->Session->write('mode', $default_mode);
				$this->set('current_mode',$default_mode);
			}
			return true;
		}
		return false;
	}
	
	
	/** 
	 * reads hospital from DB after change and writes it into Session var again
	 *  should be called after changes to hospital's data, e.g. added groups/departments etc.
	 * @return boolean true if successful, else false
	 */
	protected function updateActiveHospital($id = null) {
		if(!empty($id)) { // set it to $id

			App::uses('HospitalModel', 'Model');
			$hospital = $this->Hospital->read(null,$id);
			$accessible_hospitals = $this->Session->read('Auth.User.AccessibleHospitals');

			if(!$hospital) return false;

			// is $id an accessible hospital's id?
			$temp_array = empty($accessible_hospitals)? array() :	array_keys($accessible_hospitals);
			if(!in_array($id, $temp_array))
				return false;
			//all ok, we can write it to the variables...
			$this->current_hospital = $hospital;
			$this->Session->write('current_hospital', $hospital);

		} else { // $id is empty, just update the variable

			// set $this->current_hospital for all controllers from Session var
			$this->current_hospital = $this->Session->read('current_hospital');
			
		}
		// make it available in views as well
		$this->set('current_hospital', $this->current_hospital);
	}
	
	
	/**
	 * updates accessible hospitals list
	 * should be called after each hospital change
	 * @return true if successful, else false
	 */
	protected function updateAccessibleHospitals() {
		//if nobody is logged in, cancel
		if(!$this->Auth->user('id')) return false;

		$hospitals = $this->Hospital->find('list'); //FIXME: only the ones I have access to
		$this->Session->write('Auth.User.AccessibleHospitals', $hospitals);
	}
	
	
	/**
	 * updates user's grants in Session var
	 * @return true if successful, else false
	 */
	protected function updateUserGrants() {
		//if nobody logged in, cancel
		$user_id = $this->Auth->user('id');
		if(!$user_id) return false;

		$grants = array();
		$user = $this->User->read(null, $user_id);
		if($user) foreach($user['Group'] as $group) {
			
			// TODO: change this UUID to a 'constant', maybe Configure:read('Critecia.Admin_UUID') ?
			if($group['id'] === '4dced6f4-65cc-41b8-afcf-1305771e7925') { // internal SuperAdmins group
				$grants = 'all';
				break;
			}
			$tempgrant = $this->Grant->findByGroupId();
			if(!$tempgrant) continue;
			if(!$tempgrant['Grant']['hospital_id']) { // is a global action, so no hospital_id given
				$grants[0][] = $tempgrant['Grant']['name']; // [0] means global, "no hospital"
			} else { // hospital specific action
				$grants[$tempgrant['Grant']['hospital_id']][] = $tempgrant['Grant']['name'];
			}
		}
		$this->Session->write('Auth.User.Grant', $grants);
	}
}