<?php
/**
 *    This file is part of Critecia
 *    Copyright (C) 2011 Alexander Lang & Christian A. Reiter
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Affero General Public License as
 *    published by the Free Software Foundation, either version 3 of the
 *    License, or (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Affero General Public License for more details.
 *
 *    You should have received a copy of the GNU Affero General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

class AccessControlComponent extends Component {
	public $components = array('Auth', 'Session');
	
	/**
	 * checks the current logged in user against the given right
	 * 
	 * @param $right string Access right in the form "Controller/action"
	 * @return boolean true if action is granted, else false
	 * 
	 * grants should be in this form:
	 * grant = array(
	 * 		Hospital_UUID => array(
	 * 			Controller => array(
	 * 				0 => action1,
	 * 				1 => action2,
	 * 				...
	 * 			)
	 * 		),
	 * 		Hospital_UUID => ...
	 * 	);
	 * 
	 * e.g:
	 * grants = array(
	 * 	// global grants
	 * 		[0] => array(
	 * 			'Users/global_add'					
	 * 			),
	 * 		[4dbaa344-2a18-476a-9039-059d771e7925] => array(
	 * 			'Users/index',
	 * 			'Users/view',
	 * 			'Groups/index'
	 * 			),
	 * 		[4dbaa344-2a18-476a-9339-059d771e7925] => array(
	 * 			...
	 * 			);
	 */

	function check($right) {
	
		# WHEN HAVING PROBLEMS WITh GRANTS UNCOMMENT THIS
		# e.g. User "admin" was kicked out of "SuperAdmins" group etc.
		 return true;
		
		if($this->Auth->user()){
			// get grants of all groups the user is in
			$grants = $this->Session->read('Auth.User.Grant');
			if(!$grants) return false;
			if($grants == 'all') { // SuperAdmins group 
				return true;
			}
			// if queried right is in grants of current user then all is ok
			if(isset($this->params['prefix']) && $this->params['prefix'] == 'global') { // global action
			
				if(in_array($right, $grants[0])) return true;
				
			} else { // hospital specific action
				$current_hospital = $this->Session->read('current_hospital');

				// no hospital set, but calling hospital specific action
				if(!$current_hospital) return false;
				
				// allowed action?
				if(isset($grants[$current_hospital['Hospital']['id']]) && in_array($right, $grants[$current_hospital['Hospital']['id']])) return true;
			}
		} 						
		// everything else returns false
		return false;
	}
		

	function grant($group_id, $right, $hospital_id = null) {
		$this->Group = ClassRegistry::init('Group');
		$group=$this->Group->read(null, $group_id);
		if(!$group) return false;
		if($right != 'all') {
			$this->Right = ClassRegistry::init('Right');
			$right = $this->Right->findByName($right);
			if(!$right) return false;
			
			$this->Grant = ClassRegistry::init('Grant');
			$this->Grant->create();
			$this->data['Grant'] = array(
				'hospital_id' => $hospital_id,
				'group_id' => $group_id,
				'right_id' => $right['id'],
			);
			if(!$this->Grant->save($this->data)) return false;
		}
		return true;
	}
	
}
?>
	