<?php
 /**
 *    UsersController.php - This file is part of Critecia
 *    Copyright (C) 2011 Alexander Lang & Christian A. Reiter
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Affero General Public License as
 *    published by the Free Software Foundation, either version 3 of the
 *    License, or (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Affero General Public License for more details.
 *
 *    You should have received a copy of the GNU Affero General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

class UsersController extends AppController {
	public $scaffold;
	public $uses = array('User', 'Hospital', 'Group');
	
	public function beforeFilter() {
		parent::beforeFilter();
// 		$this->Auth->allow('global_newpwd');
	}
	
	/**
	 * logs  user in
	 * @return void
	 */
	public function global_login() {
		if ($this->request->is('post')) {
			if ($this->Auth->login()) {
				
				$this->Session->setFlash('Willkommen.');
				// set user's groups into session variable 'Auth.User.Group'
				$user = $this->User->read(null, $this->Auth->user('id'));
				$this->Session->write('Auth.User.Group', $user['Group']);
				$this->updateUserGrants();
				
				// set user's accessible hospitals into session variable 'AccessibleHospitals'
				$hospitals = $this->Hospital->find('list'); //TODO: only the ones I have access to
				$this->Session->write('Auth.User.AccessibleHospitals', $hospitals);
				
				// switch to management mode
				$this->switchMode('system');
				
				return $this->redirect($this->Auth->redirect());
				
			} else {
				$this->Session->setFlash(('Benutzername oder Passwort ist falsch.'), 'default', array(), 'auth');
			}
		}
		$this->render('login');	
	}
	
	/**
	 * logs out current user
	 * @return void
	 */
	function global_logout() {
		$this->Session->destroy();
		$this->redirect($this->Auth->logout());
	}
	
	
	/**
	 * lists all users of the current hospital
	 * @return void
	 */
	 function index() {
		$users = $this->User->find('all',
			array('order' => 'lastname ASC, firstname ASC',
			'conditions' => array('hospital_id' =>
				$this->current_hospital['Hospital']['id'])));
		$hospital = $this->current_hospital;
		$this->set(compact('users', 'hospital'));
	}
	
	/**
	 * lists all SuperAdmin users that are not part of any hospital
	 * @param $hospital_id string UUID of the hospital, if null, the current is taken
	 * @return void
	 */
	function global_index($hospital_id = null) {
		$conditions = $hospital_id? array('conditions' => array('hospital_id' => $hospital_id)) : null;
		$users = $this->User->find('all', array('order' => 'lastname ASC, firstname ASC'), $conditions);
		$hospital = $this->Hospital->read(null, $hospital_id);
		$this->set(compact('users', 'hospital'));
		$this->render('index');
	}
	
	
	
	/**
	 * shows user, no matter which one
	 * @param $user_id string UUID of the user
	 * @return void
	 */
	function global_view($user_id) {
		$user = $this->User->read(null, $user_id);
		$this->set(compact('hospital_id'));
		$this->set('groups', $this->User->Group->find('list'));
		$this->render('view');
	}


	/**
	 * shows user, tests if I can access this one
	 * @param $user_id string given user that should be shown
	 * @return void
	 */
	function view($user_id) {
		//TODO: validate $hospital_id
		$user = $this->User->read(null, $user_id);

		// trying to x-view another user?
		//FIXME: better check if I am allowed to see this user - as it may be that I
		// can see other users of other hospitals, but am NOT SuperUser!
		// e.g. Rotationskoordinatoren...
		if(!$user || !in_array($user['User']['hospital_id'],
			array_keys($this->Session->read('Auth.User.AccessibleHospitals')))) {
			$this->Session->setFlash('Ungültiger Benutzer');
			$this->redirect($this->referer());
		}
		$this->set('groups', $this->User->Group->find('list'));
	}
	
	/**
	 * hospital specific edit function
	 * @param $user_id string given user that should be shown
	 * @return void
	 */
	public function edit($user_id) {
		if(!$user_id && empty($this->data)) {
			$this->Session->setFlash('Ungültiger Benutzer');
			$this->redirect(array('action' =>'index'));
		}
		$user = $this->User->read(null, $user_id);
		if(!$user || !in_array($user['User']['hospital_id'],
			array_keys($this->Session->read('Auth.User.AccessibleHospitals')))) { // trying to x-edit another user?
			$this->Session->setFlash('Ungültiger Benutzer');
			$this->redirect(array('action' =>'index'));
		}
		$this->set('title_for_layout','Benutzer bearbeiten');
		if (!empty($this->data)) {
			if ($this->User->saveAll($this->data)) {
				$this->Session->setFlash('Die Benutzerdaten wurde gespeichert.');
				$this->redirect(array('action'=>'index'));
			} else {
				$this->Session->setFlash('Die Benutzerdaten konnten nicht gespeichert werden.');
			}
		} else { // fresh form without data
			$this->data = $user;

			//FIXME: all admin-granted groups are lost here:
			$groups = $this->Group->find('list', array('conditions' =>
				array('hospital_id' => $user['User']['hospital_id'])));
			$hospitals = $this->Session->read('Auth.User.AccessibleHospitals');
			$this->set(compact('groups', 'hospitals'));
		}
		$this->set('action', 'edit');
		$this->render('add');
	}
	
	
	
	/** edit function to edit /me
	 * @param $user_id string user id
	 * @return void
	 */
	public function edit_own($user_id) { 
		//TODO: implement stub
		
		$this->set('action', 'edit');
		$this->render('add');
	}
	
	
	
	/** global edit function
	 * @param $user_id string user id
	 * @return void
	 */
	public function global_edit($user_id) {
		$this->set('title_for_layout','Benutzer bearbeiten');
		if(!$user_id && empty($this->data)) {
			$this->Session->setFlash('Ungültiger Benutzer');
			$this->redirect(array('action' =>'index'));
		}
		if (!empty($this->data)) {
			if ($this->User->saveAll($this->data)) {
				$this->Session->setFlash('Die Benutzerdaten wurde gespeichert.');
				$this->redirect(array('action'=>'index'));
			} else {
				$this->Session->setFlash('Die Benutzerdaten konnten nicht gespeichert werden.');
			}
		} else {
			$this->data = $this->User->read(null, $user_id);
			if(empty($this->data)) $this->cakeError('wrongID'); //TODO implement!
			$groups = $this->User->Group->find('list');
			$hospitals = $this->Session->read('Auth.User.AccessibleHospitals');
			$this->set(compact('groups', 'hospitals'));
		}
		$this->set('action', 'edit');
		$this->render('add');
	}

	/** global add function
	 * @param $hospital_id string hospital in which the user should be created. if null, create an admin user
	 * @return void
	 */
	public function global_add($hospital_id = null) {
		$this->set('title_for_layout','Benutzer anlegen');
		if (!empty($this->data)) {
			$this->User->create();
			if ($this->User->saveAll($this->data)) {
				$this->Session->setFlash('Der Benutzer wurde angelegt.');
				$this->redirect(array('action'=>'index'));
			} else {
				$this->Session->setFlash('Der Benutzer konnte nicht angelegt werden.');
			}
		} else {
			$hospitals = $this->Hospital->find('list');
			$groups = $this->Group->find('list'); //FIXME: only ones from selected hospital
			$this->set(compact('groups', 'hospitals'));
		}
		$this->set('action', 'add');
		$this->render('add');
	}


	/** hospital specific add function
	 * @param $hospital_id string hospital in which the user should be created. if null, create an admin user
	 * @return void
	 */
	public function add() {
		$this->set('title_for_layout','Benutzer anlegen');
		if (!empty($this->data)) {
			$this->User->create();
			if ($this->User->saveAll($this->data)) {
				$this->Session->setFlash('Der Benutzer wurde angelegt.');
				$this->redirect(array('action'=>'index'));
			} else {
				$this->Session->setFlash('Der Benutzer konnte nicht angelegt werden.');
			}
		} else {
			$groups = $this->Group->find('list'); //FIXME only accessible!
			$hospitals = $this->Session->read('Auth.User.AccessibleHospitals');
			$this->set(compact('groups', 'hospitals'));
		}
		$this->set('action', 'add');
	}
	
	
//TODO: what about list/grid of users with all data (phones, email, photo etc)
	
	/** shows a list of email addresses of users
	 * @return void
	 */
	function showEMails() { // zeigt eine eMail-Liste aller Turnusärzte an
		//TODO: constrain: only emails, only from this hospital
		//TODO: do we really need this function?
		$users = $this->User->find('all', array('order' => 'lastname ASC, firstname ASC'));
		$this->set('users', $users);
	}

	/** shows a list of phone numbers of users
	 * @return void
	 */
	function showPhones() {
	//TODO: constrain: only  phones, only from this hospital
		$users = $this->User->find('all', array('order' => 'lastname ASC, firstname ASC'));
		$this->set('users', $users);
	}

	/** shows a list of photos of users
	 * @return void
	 */
	function showPictures() {
	//TODO: constrain: only pictures, only from this hospital
		$users = $this->User->find('all',
						array('order' => 'lastname ASC, firstname ASC',
							  'conditions' => array('picture IS NOT NULL')));
		$this->set('users', $users);
	}

	/** shows a list of birthdays of users
	 * @return void
	 */
	function showBirthdays() {
	//TODO: constrain: only birthdays, only from this hospital
		$users = $this->User->find('all',
			array('fields' => array('firstname', 'lastname',
							'birthdate',
							'DATE_FORMAT(birthdate,"%m") AS month'),
				  'order' => 'month ASC',
				  'conditions' => array('birthdate IS NOT NULL')));
		$this->set('users', $users);
	}

/**************************  delete   **************************/

	/**
	 * deletes user, checks first if we are allowed to do that
	 * @param $user_id string UUID of user to delete
	 * @return void
	 */
	public function delete($user_id)
	{
		if (!$user_id) {
			$this->Session->setFlash('Ungültige Benutzer-ID.');
			$this->redirect($this->referer());
		}
		$user = $this->User->read(null, $user_id);
		if(!$user || !in_array($user['User']['hospital_id'],
			array_keys($this->Session->read('Auth.User.AccessibleHospitals')))) {
			$this->Session->setFlash('Kein Zugriff auf dieses Krankenhaus');
			$this->redirect($this->referer());
		}
		if ($this->User->delete($user_id)) {
			$this->Session->setFlash('Benutzer wurde gelöscht.');
			$this->redirect($this->referer());
		}
		$this->Session->setFlash('Benutzer konnte nicht gelöscht werden.');
		$this->redirect($this->referer());
	}

	/**
	 * deletes user (as superuser)
	 * @param $user_id string UUID of user to delete
	 * @return void
	 */
	public function global_delete($user_id)
	{
		if (!$user_id) {
			$this->Session->setFlash('Ungültige Benutzer-ID.');
			$this->redirect($this->referer());
		}
		if ($this->User->delete($user_id)) {
			$this->Session->setFlash('Benutzer wurde gelöscht.');
			$this->redirect($this->referer());
		}
		$this->Session->setFlash('Benutzer konnte nicht gelöscht werden.');
		$this->redirect($this->referer());
	}
	
}
?>