<?php
 /**
 *    Vacation.php - This file is part of Critecia
 *    Copyright (C) 2011 Alexander Lang
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Affero General Public License as
 *    published by the Free Software Foundation, either version 3 of the
 *    License, or (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Affero General Public License for more details.
 *
 *    You should have received a copy of the GNU Affero General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

	class Vacation extends AppModel {	
		public $belongsTo = array('VacationType', 'User');
		
		public $validate = array(
				'from' => array(
						'checkBegindate_compare2until' => array(
							'rule' => array('checkBegindate_compare2until','until'),
							'message' => 'Der Urlaubsbeginn kann nicht nach dem Urlaubsende liegen.'
							),
						'checkBegindate_4future' => array(
							'rule' => array('checkBegindate_4future'),
							'message' => 'Der Urlaubsbeginn kann nicht in der Vergangenheit liegen.'
							)
						),
				'until' => array(
						'rule' => 'date',
						'message' => 'Das Urlaubsenddatum wurde nicht korrekt übergeben.'
						),
				'createdfrom' => array(
						'checkCreatedfrom' => array(
							'rule' => array('checkCreatedfrom','from'),
							// Diese Nachricht kann nicht dargestellt werden, weil das 
							// Formfeld 'CreatedFrom' HIDDEN ist...
							'message' => 'Der Urlaub kann nicht mehr als 7 Tage vom ursprünglichen eingetragenen Datum verschoben werden.'
							)
						),
				'createduntil' => array(
						'checkCreateduntil' => array(
							'rule' => array('checkCreateduntil','until'),
							// Diese Nachricht kann nicht dargestellt werden, weil das 
							// Formfeld 'CreatedUntil' HIDDEN ist...
							'message' => 'Der Urlaub kann nicht mehr als 7 Tage vom ursprünglichen eingetragenen Datum verschoben werden.'
							)
						),
				'vacation_type_id' => array(
						'rule' => array('range',0,4), // erlaubt ist also eine Zahl zwischen 1-3
						'message' => 'Es wurde eine inkorrekte Urlaubsart übergeben.'
						),
				'comment' => array(
				        'allowedCharacters'=> array(
							'rule' => '|^[a-zA-Z0-9öäüÖÄÜß\(\)\.\-\:\,\;\!\?\%\&\=\*\# ]*$|',
							'message' => 'Hier sind nur Buchstaben, Zahlen und die Sonderzeichen ",.-;:()*#!?%&=" erlaubt.'
            				),
            			'checkComment' => array(
							'rule' => array('checkComment'),
							'message' => 'Wenn du einen Fortbildungs- bzw. Sonderurlaub eingibst, mußt du bitte einen beschreibenden Kommentar eingeben (z.B. "Notarztkurs Bad Gastein").'
							)
						)
			);
			
		public function checkBegindate_compare2until($data, $field) {
			$valid = false;
			
			$date = DateTime::createFromFormat('Y-m-d', $data['from']);
			$date_bis = DateTime::createFromFormat('Y-m-d', $this->data[$this->name][$field]);		
			if ($date <= $date_bis)
				$valid = true;

			return $valid;
		}

		public function checkBegindate_4future($data) {
			$valid = false;
			
			$date = DateTime::createFromFormat('Y-m-d', $data['from']);
			$date_today = new DateTime();		
			if ($date >= $date_today)
				$valid = true;

			return $valid;
		}

		public function checkCreatedfrom($data, $field) {	
			$valid = $this->checkDateDifference($data['createdfrom'], $this->data[$this->name][$field], 7);
			return $valid;
		}
		
		public function checkComment($data) {	
			if (($this->data[$this->name]['vacation_type_id'] == 2) OR ($this->data[$this->name]['vacation_type_id'] == 3)) {
				if ($data['comment'] != '')
					$valid = true;
				else
					$valid = false;
			} else
				$valid = true;
			return $valid;
		}
		
		public function checkCreateduntil($data, $field) {	
			$valid = $this->checkDateDifference($data['createduntil'], $this->data[$this->name][$field], 7);
			return $valid;
		}
		
		public function checkDateDifference($date1, $date2, $number_of_days) {
		// überprüft, ob die Zeitspanne zwischen den beiden übergebenen Datumsangaben
		// (date1, date2) mit der maximalen Zeitspanne (number_of_days) übereinstimmt.
		// liefert TRUE wenn die Zeitspanne passt, bzw. FALSE wenn sie überschritten wird.
		// date1, date2 erwartet eine Eingabe im Format 'Y-m-d'
		// number_of_days erwartet die Anzahl der Tage als INT value
			$valid = false;
			
			$date = DateTime::createFromFormat('Y-m-d', $date1);
			$check = DateTime::createFromFormat('Y-m-d', $date2);	
			
			$interval = $date->diff($check);
			$diff = $interval->format('%a');
			
			if ($diff <= $number_of_days)
				$valid = true;

			return $valid;			
		}
						
		
						
	}
	
?>
 
