<?php
/**
 *    This file is part of Critecia
 *    Copyright (C) 2011 Alexander Lang & Christian A. Reiter
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Affero General Public License as
 *    published by the Free Software Foundation, either version 3 of the
 *    License, or (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Affero General Public License for more details.
 *
 *    You should have received a copy of the GNU Affero General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


class AccessControlHelper extends AppHelper {
	public $helpers = array('Session');
	
	
	/**
	 * checks the current logged in user against the given right
	 * @param $right string Access right in the form "Controller/action"
	 * @return boolean if action is granted or not
	 */
	function check($right) {
		
		# WHEN HAVING PROBLEMS WITh GRANTS UNCOMMENT THIS
		# e.g. User "admin" was kicked out of "SuperAdmins" group etc.
		// return true;
		
		
		// get grants of all groups the user is in
		$grants = $this->Session->read('Auth.User.Grant');
		if(!$grants) return false;
		if($grants == 'all') { // SuperAdmins group 
			return true;
		}
		
		// if queried right is in grants of current user then all is ok
		if(isset($this->params['prefix']) && $this->params['prefix'] == 'global') { // global action
				
			if(in_array($right, $grants[0])) return true;
				
		} else { // hospital specific action
			$current_hospital = $this->Session->read('current_hospital');
		
			// no hospital set, but calling hospital specific action
			if(!$current_hospital) return false;
					
			// allowed action?
			if(isset($grants[$current_hospital['Hospital']['id']]) && in_array($right, $grants[$current_hospital['Hospital']['id']])) return true;
		}
		// everything else returns false
		return false;
	}
}
?>
